#include "player.h"
#include "card.h"

#include <algorithm>
#include <iostream>
#include <ostream>


#include "randIntClass.h"


using namespace std;


pair<int, int> openingBid;
int opener;
pair<int, int> lastBid;
player players [4];

void announce(int player, pair<int, int> bid, bool friendly = true)
{
     for(int cur_player = 0; cur_player < 4; ++cur_player)
             players[cur_player].infer(player, bid, openingBid, friendly);
}
void announce_opener(int player)
{
     for(int cur_player = 0; cur_player < 4; ++cur_player)
             players[cur_player].get_knowledge().setOpener(player);
}

void bid( player* players )
{
    int passes_in_row = 0;
     
    int curPlayer = player::North;
    
    // wait for opening bid:
    for(; curPlayer <= player::West; ++curPlayer)
    {
        if( (openingBid = players[curPlayer].openBid()).first != 0 )
        {     
            lastBid       = openingBid;
            opener        = curPlayer;
            announce_opener(opener);
            passes_in_row = 0;
            announce(curPlayer, openingBid);
            break;
        }
        else
        {
            announce(curPlayer, pair<int,int> (0,0) );
            ++passes_in_row;
        }
    }
    
    // wait for continuing bidding
    while( passes_in_row < 3 )
    {
        pair<int, int> possible_lastBid;
        // oppose bid 
        if( (possible_lastBid = players[((++curPlayer)%4)].opposeBid(openingBid, lastBid)).first != 0 )   
        {
            if(possible_lastBid.first != 0)
                  lastBid = possible_lastBid;
            passes_in_row = 0;
            announce( (curPlayer%4), lastBid, false );
        }
        else
        {
            ++passes_in_row;
            announce( (curPlayer%4), pair<int,int> (0,0), false );
        }
        
        if( passes_in_row >= 3)
            break;        

        //friendly bid
        if( (possible_lastBid = players[((++curPlayer)%4)].friendlyBid(openingBid, lastBid)).first != 0 )  
        { 
            if(possible_lastBid.first != 0)
                lastBid = possible_lastBid;            
            passes_in_row = 0;
            announce( (curPlayer%4), lastBid);
            break; // because only opening and response are implemented
        }
        else
        {
            ++passes_in_row;
            announce( ((curPlayer)%4), pair<int,int> (0,0) );            
        }
    }
}


int main()
{

    vector <card>deck;
    vector <pair<int, int> > placedBids; // 35 is the maximum of bids possible in a game

        
    // init players
    for(int i = 0; i < 4; ++i)
        players[i] = player(i);
    

    for(int num = card::Two; num <= card::A; ++num){
        for(int suit = card::Clubs; suit <= card::Spades; ++suit)
            deck.push_back( card(num, suit) );
    }    

    // shuffle cards
    RandIntClass Rand;
    std::random_shuffle(deck.begin(), deck.end(), Rand );
    
    // deal cards
    unsigned card = 0;
    string hands[4];
    while(card < deck.size() )
    {
        for(int pl = 0; pl < 4; ++pl)
        {
            switch(deck[card].getValue())
            {
                  case card::A:
                       hands[pl].append(1,'A');
                       break;
                  case card::K:    
                       hands[pl].append(1,'K');
                       break;
                  case card::Q:    
                       hands[pl].append(1,'Q');
                       break;
                  case card::J:
                       hands[pl].append(1,'J');
                       break;
            }    
            players[pl].addCard(deck[card++]);
        }
    }

    // evaluate hands
    for(int i = 0; i < 4; ++i)
    {
        players[i].evaluateHand();    
        
        // set own points
        (players[i].get_knowledge()).set(players[i].get_id(), players[i].getPoints());
        
        // limit max-points for other players
        for(int j = 0; j < 4; ++j)
            players[i].get_knowledge().update_range(j, 0, 40 - players[i].getPoints());
    }
    
    
        
        
    //print hands
    cout << "\n\ncards held by north:" << endl;
    players[0].printTo(cout);
    cout << "is worth: " << players[0].getPoints() << " points\n" << endl;
    cout << "cards held by east:" << endl;
    players[1].printTo(cout);
    cout << "is worth: " << players[1].getPoints() << " points\n" << endl;   
    cout << "cards held by south:" << endl;
    players[2].printTo(cout);
    cout << "is worth: " << players[2].getPoints() << " points\n" << endl;
    cout << "cards held by west:" << endl;
    players[3].printTo(cout);
    cout << "is worth: " << players[3].getPoints() << " points\n" << endl;
        
    cout << "bids are started by north:\n\n";
  
    bid(players);
    
    
//printing knowledge of players:
    cout << "printing knowledge of players:" << endl;       
    for (int i = 0; i < 4; ++i)
    {
        cout << "\nplayer " << i << " knows:" << endl;
        players[i].get_knowledge().print();                              
    }
    
    vector<string> doorsnede;
    
    int highestK = 0;
    for (int i = 0; i < 4; ++i)
    {
        if (players[highestK].get_knowledge().combinedK().size() > players[i].get_knowledge().combinedK().size())
           highestK = i;
    }
    cout << "speler " << highestK << " knows most!\n";
    vector<string> lowest;
    copy(players[highestK].get_knowledge().combinedK().begin(),players[highestK].get_knowledge().combinedK().end(),back_inserter(lowest));
    
    for (vector<string>::iterator it = lowest.begin(); it < lowest.end(); it++)
    {
        int found(0);
        for (int i = 0; i < 4; i++)
        {
            string verdeling(*it);
            if (
                      find (players[i].get_knowledge().combinedK().begin(), 
                            players[i].get_knowledge().combinedK().end(), 
                            verdeling)
                            !=
                            players[i].get_knowledge().combinedK().end())
            {
                   found++;
            }
            
        }
      
        if (found == 4)
           doorsnede.push_back(*it);
    }
    
    cout << "\nDistributed knowledge(the solution):\n";
    unique_copy(doorsnede.begin(), doorsnede.end(), ostream_iterator<string>(cout, " "));
    
    return 0;
    
}





