#ifndef _PLAYER_H_
#define _PLAYER_H_

#include <string>
#include <vector>
#include <ostream>
#include "card.h"
#include <iostream>
#include <algorithm>
#include "knowledge.h"

#include <iosfwd>

using namespace std;

class player
{
      knowledge       d_knowledge;
      vector <card>   d_cards;
      int             d_id;
      int             d_points;
      bool            d_NTDistribution;
      pair<int, int>  suitfrequencies[4]; // fist = number of cards, second is suit
                                          // sorted from short suits to long suits
      pair<int, int>  d_lastBid;
      
    public: 
        
      enum PlayerName{North, East, South, West};
            
      player()
      :
      d_knowledge(-1)
      {}
       
      player(int i) // i=0 for north, i=1 for east, i=2 for south, i=3 for west
      :
          d_knowledge(i),
          d_id(i),
          d_points(0),
          d_NTDistribution(false)
      {}
      
      knowledge &get_knowledge()
      {
           return d_knowledge;
      }
      int get_id()
      {
           return d_id;
      }
      void addCard(card c)
      {
           d_cards.push_back(c);
      }
      
      void evaluateHand();
      pair<int, int> bid(vector <pair<int, int> > bids);
      void printSuit(ostream &ostr, int suit);
      pair<int, int> printBid(ostream &ostr, int num, int suit);
      
      void infer(int player, pair<int,int> bid, pair<int,int> openingBid, bool friendly);

      ostream& printTo(ostream &ostr)
      {
          for(unsigned idx = 0; idx < d_cards.size(); ++idx)
              d_cards[idx].printTo(ostr);
          return ostr << "\n";
      }
      
      bool checkFreq( int num, int suit)
      {
          for(int idx = 0; idx < 4; ++idx)
          {
              if(suitfrequencies[idx].first == num && suitfrequencies[idx].second == suit)
                  return true;
          }
          return false;
      }
      
      int getPoints()
      {
          return d_points;
      }
    
      pair<int, int> openBid(); 
      pair<int, int> opposeBid( pair<int, int>, pair<int, int> );
      pair<int, int> friendlyBid( pair<int, int>, pair<int, int> );
      
   private:
      void printPlayer();
};

#endif
